%     This file is part of correlation_gui.
% 
%     correlation_gui is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     correlation_gui is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with correlation_gui.  If not, see <http://www.gnu.org/licenses/>.
% 
%     Diese Datei ist Teil von correlation_gui.
% 
%     correlation_gui ist Freie Software: Sie knnen es unter den Bedingungen
%     der GNU General Public License, wie von der Free Software Foundation,
%     Version 3 der Lizenz oder jeder neueren verffentlichten Version, 
%     weiter verteilen und/oder modifizieren.
% 
%     correlation_gui wird in der Hoffnung, dass es ntzlich sein wird, aber
%     OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
%     Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
%     Siehe die GNU General Public License fr weitere Details.
% 
%     Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
%     Programm erhalten haben. Wenn nicht, siehe <https://www.gnu.org/licenses/>.

function [K, zValues] = calculateK(scatterVector, absIntensity, C1, C2, sigma)
% calculateK -  computes the values of the correlation function K''(z) 
%                   
% 
% Syntax:  K = calculateK(scatterVector, absIntensity,, C1, C2, sigma)
%
% Inputs:
%    scatterVector - used values of the scatter vectors in 1/nm 
%    absIntensity - absolute intensities in e.u./nm^3 corresponding to the
%                   scatter vectors
%    P,c1,c2 - parameters of a given Porod Fit (I = P*s^(-4) + c1*s^2 + c2)
%    sigma - width of the Gaussian window function 
%               exp((2*pi*sigma.*scatterVector)^2)
%
% Outputs:
%    K - K(z) 
%    zvalues - points z at which K(z) is evaluated

% Other m-files required: Window Function
% Subfunctions: none
% MAT-files required: none

% Known bugs: none
%
% Author: Susanne Pieschner
% Last revision: 11-June-2015

%%
% Calculation of K(z):
% Calculate g(s_i) = 4*pi*(I(s_i)-c2)*s_i^2 - c1*s^4, for s_i \in scatterVector
% Use linear interpolation to obtain equidistant data points and
% Extrapolation to the left: Set g(0) = 0
% and use linear interpolation to determine 
%           g(s_i), for 0 < s_i < min(s_i | s_i \in  scatterVector) 
minDist = min(diff(scatterVector));
integrandOfK = 4 * pi * ((absIntensity -C2).* scatterVector.^2 ...
                        - C1.*scatterVector.^2);
extrapolatedScatterVector = (0:minDist:scatterVector(end))';
extrapolatedIntegrandOfK = interp1([0 scatterVector'], ...
    [0 integrandOfK'], extrapolatedScatterVector, 'linear');

% Multiply g by the window function wf(s_i) = exp(4*pi^2*sigma^2*s_i^2)
% K''(z) = 4*pi * int_0^s_max [I(s)*s^2 *exp(-4*pi^2*sigma^2*s^2)] ds
%       = minDist/2 * DCT(g(s)*wf(s)]
windowFcnValues = windowFunction(extrapolatedScatterVector,sigma);


% Set transform length to the next but one power of 2 (bernchste
% Zweierpotenz) of the length of the integrand
nFFT = pow2(nextpow2(length(extrapolatedIntegrandOfK))+1); 

K = minDist * real(fft(extrapolatedIntegrandOfK .* windowFcnValues, nFFT));

% only the first half of the resulting vector is relevant because the
% second half mirrors the first half
K = K(1:end/2);

zValues = (1/minDist * linspace(0,1/2,nFFT/2))'; 




