%     This file is part of correlation_gui.
% 
%     correlation_gui is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     correlation_gui is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with correlation_gui.  If not, see <http://www.gnu.org/licenses/>.
% 
%     Diese Datei ist Teil von correlation_gui.
% 
%     correlation_gui ist Freie Software: Sie knnen es unter den Bedingungen
%     der GNU General Public License, wie von der Free Software Foundation,
%     Version 3 der Lizenz oder jeder neueren verffentlichten Version, 
%     weiter verteilen und/oder modifizieren.
% 
%     correlation_gui wird in der Hoffnung, dass es ntzlich sein wird, aber
%     OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
%     Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
%     Siehe die GNU General Public License fr weitere Details.
% 
%     Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
%     Programm erhalten haben. Wenn nicht, siehe <https://www.gnu.org/licenses/>.

function K2Zero = calculateK2AtZero(scatterVector, absIntensity, ...
                    sigma, P, C1, C2)
% calculateK2AtZero -  computes the value of the interface distribuition 
%                       function K''(z) at z=0
%                   
% 
% Syntax:  K2Zero = calculateK2AtZero(scatterVector, absIntensity, P, c1, ...
%                    c2, sigma)
%
% Inputs:
%    scatterVector - used values of the scatter vectors in 1/nm 
%    absIntensity - absolute intensities in e.u./nm^3 corresponding to the
%                   scatter vectors
%    P,c1,c2 - parameters of a given Porod Fit (I = P*s^(-4) + c1*s^2 + c2)
%    sigma - width of the Gaussian window function 
%               exp((2*pi*sigma.*scatterVector)^2)
%
% Outputs:
%    K2Zero - K''(0) 

% Other m-files required: Window Function
% Subfunctions: none
% MAT-files required: none

% Known bugs: none
%
% Author: Susanne Pieschner
% Last revision: 21-May-2015

%%
% Calculation of K''(0):
% Calculate f(s_i) = P - (I(s_i)-c2)*s_i^4 +c1*s_i^6, for s_i \in scatterVector
% Use linear interpolation to obtain equidistant data points and
% Extrapolation to the left: Set f(0) = P
% and use linear interpolation to determine 
%           f(s_k), for 0 < s_k < min(s_i | s_i \in scatterVector) 
minDist = min(diff(scatterVector));
integrandOfK2 = P - (absIntensity - C2).*scatterVector.^4 ...
                    + C1*scatterVector.^6;
extrapolatedScatterVector = (0:minDist:scatterVector(end))';
extrapolatedIntegrandOfK2 = interp1([0 scatterVector'], ...
    [P integrandOfK2'], extrapolatedScatterVector, 'linear');


% Multiply f by the window function wf(s_i) = exp(4*pi^2*sigma^2*s_i^2)
% K''(0) = 16*pi^3* int_0^s_max [P - (I(s)-c2)*s^4 + c1*s^6]*exp(-4*pi^2*sigma^2*s^2) ds
% .... = 16*pi^3* DeltaS /2 *[f(0)*wf(0) + f(s_max)*wf(s_max) ...
%            + 2* (sum_{s_1 to s_max-1} f(s)*wf(s))]
windowFcnValues = windowFunction(extrapolatedScatterVector,sigma);
K2Zero = 16 * pi^3 * minDist ...
   *( 1/2 * ( extrapolatedIntegrandOfK2(1) .* windowFcnValues(1) ...
            + extrapolatedIntegrandOfK2(end) .* windowFcnValues(end)) ...
   + sum( extrapolatedIntegrandOfK2(2:end-1) .* windowFcnValues(2:end-1)));
 


