%     This file is part of correlation_gui.
% 
%     correlation_gui is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     correlation_gui is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with correlation_gui.  If not, see <http://www.gnu.org/licenses/>.
% 
%     Diese Datei ist Teil von correlation_gui.
% 
%     correlation_gui ist Freie Software: Sie knnen es unter den Bedingungen
%     der GNU General Public License, wie von der Free Software Foundation,
%     Version 3 der Lizenz oder jeder neueren verffentlichten Version, 
%     weiter verteilen und/oder modifizieren.
% 
%     correlation_gui wird in der Hoffnung, dass es ntzlich sein wird, aber
%     OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
%     Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
%     Siehe die GNU General Public License fr weitere Details.
% 
%     Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
%     Programm erhalten haben. Wenn nicht, siehe <https://www.gnu.org/licenses/>.

function [scatterVector, absIntensity, desmearedError] = ...
              loadCustomFile(fileName, pathName, nHeaderLines, delimiter, ...
              scattVectType, scattVectUnit, colScattVect, colAbsInt, colError)
% loadCustomFile -    Extracts the scatter vectors and the corresponding  
%                       absolute intensities and desmeared errors (optional)
%                       from the specified data file
%
% Syntax:  [scatterVector,absIntensity, desmearedError] = ...
%                  loadCustomFile(fileName, pathName, nHeaderLines,... 
%                   delimiter, colScattVect, colAbsInt, colError)
%
% Inputs:
%    fileName - name of the scatter data file
%    pathName - path of the scatter data file
%    nHeaderLines - number of header lines in the file
%    delimiter - delimiter between data columns
%    scattVectType - type of scattering vector: either 'q' or 's'
%    scattVectUnit - unit of scattering vector: either 'Angstrom' or 'nm'
%    colScattVect - column number containing the scatter vectors
%    colAbsInt -  column number containing the absolute intensities
%    colError - column number containing the desmeared errors
%
% Outputs:
%    scatterVector - scatter vectors in 1/nm from the data file
%    absIntensity - absolute intensities in e.u./nm^3 corresponding to the
%                   scatter vectors
%    desmearedError - (optional: empty if colError is NaN)
%

% Other m-files required: none
% Subfunctions: comma2point()
% MAT-files required: none
%
% Variables: 	
%
% Known bugs: none
%
% Author: Susanne Pieschner
% Last revision: 15-July-2015

%%

% change commas to points in the input file and save the result under the newFilename
Name = textscan(fileName,'%s%s','delimiter','.');
% create the name of the modified file by inserting '_Modified.' in front 
% of the filename extension
pointPositions = strfind(fileName,'.');
newFileName = [fileName(1:pointPositions(end)) '_Modified', fileName(pointPositions(end):end)];
comma2point(pathName, fileName, newFileName);

try % try to use the dataimport-function with the specified delimiter and
    % number of header lines
    
    % open data file and import contents
    [impData, delimiterOut, nHeaderLinesOut]= ...
            importdata([pathName newFileName], delimiter, nHeaderLines);
        % check whether the number of columns loaded is at least as big as
        % the highest column number specified by the user
    if (size(impData.data,2) < max([colScattVect,colAbsInt,colError]))
        error('errMsg:wrongFormat',['The selected data file does not contain'...
                'the required format!'])
    else
        scatterVector = impData.data(:,colScattVect);
        absIntensity = impData.data(:,colAbsInt);
        if (isnan(colError))
            desmearedError = [];
        else
            desmearedError = impData.data(:,colError);
        end
    end
catch ME
    try % try to load the data determining the delimiter and number of
        % header lines automatically
        
        % open data file and import contents
        [impData, delimiterOut, nHeaderLinesOut]= ...
                                    importdata([pathName newFileName]);
        % check whether the number of columns loaded is at least as big as
        % the highest column number specified by the user
        if (size(impData.data,2) < max([colScattVect,colAbsInt,colError]))
            error('errMsg:wrongFormat',['The selected data file does not contain'...
                    'the required format!'])
        else
            scatterVector = impData.data(:,colScattVect);
            absIntensity = impData.data(:,colAbsInt);
            if (isnan(colError))
                desmearedError = [];
            else
                desmearedError = impData.data(:,colError);
            end
        end
    catch ME
        error('errMsg:wrongFormat',['The selected data file does not contain'...
                    'the required format!']);
    end
end


% if the delimiter and the number of header lines that the function
% IMPORTDATA() recognized are not the same as those specified by the user,
% write the loaded data in a test file, warn the user and ask him to check
% whether the test file actually contains the data he wants to analyze;
% the test file is deleted immediately after the user selects 'OK' in the
% warning dialog box
if(~strcmp(delimiter,delimiterOut) || nHeaderLinesOut ~= nHeaderLines )
    
    if (isnan(colError))
        dlmwrite([pathName , char(strcat( Name{1}, '_test.', Name{2}))],...
            [scatterVector absIntensity],'delimiter','\t', 'precision', 10);
    else
        dlmwrite([pathName , char(strcat( Name{1}, '_test.', Name{2}))],...
            [scatterVector absIntensity desmearedError],'delimiter','\t',...
            'precision', 10);
    end
    
    message = cell(4,1);
    message{1} = 'Please check whether that data in file ';
    message{2} = ['''' pathName , char(strcat( Name{1}, '_test.', Name{2})) ''''];
    message{3} = 'is really the data you want to analyze! (before selecting ''OK'')';
    message{4} = 'If not so, select a new file or try to load the file again!';
    h = msgbox(message,'Warning','warn');
    uiwait(h)% wait until error dialog is closed, then continue
    % delete the test file
    delete([pathName , char(strcat( Name{1}, '_test.', Name{2}))]);
end    
    
% delete the modified file, where commas were changed to points    
delete([pathName newFileName]);

% convert scattering vector into proper type (s) and unit (nm)
if (strcmp(scattVectUnit,'Angstrom'))
    scatterVector = scatterVector*10;
end
if (strcmp(scattVectType,'q'))
    scatterVector = scatterVector/2/pi;
end

end


  function comma2point(pathName, fileName, newFileName)
% Generate a new file named Oldfilename_Modified.ending. In the new File all
% ',' are changed to '.' 
% Needs a full datapath and filename as input.
   
    copyfile( [pathName fileName], [pathName newFileName]);
    file    = memmapfile([pathName newFileName],'Writable',true);
    comma   = uint8(',');
    point   = uint8('.');
    file.Data(( file.Data==comma)' ) = point;
end 