%     This file is part of correlation_gui.
% 
%     correlation_gui is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     correlation_gui is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with correlation_gui.  If not, see <http://www.gnu.org/licenses/>.
% 
%     Diese Datei ist Teil von correlation_gui.
% 
%     correlation_gui ist Freie Software: Sie knnen es unter den Bedingungen
%     der GNU General Public License, wie von der Free Software Foundation,
%     Version 3 der Lizenz oder jeder neueren verffentlichten Version, 
%     weiter verteilen und/oder modifizieren.
% 
%     correlation_gui wird in der Hoffnung, dass es ntzlich sein wird, aber
%     OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
%     Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
%     Siehe die GNU General Public License fr weitere Details.
% 
%     Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
%     Programm erhalten haben. Wenn nicht, siehe <https://www.gnu.org/licenses/>.

function [scatterVector,absIntensity, desmearedError] = ...
              loadStandardFile(fileName, pathName)
% loadStandardFile -    Extracts the scatter vectors and the corresponding  
%                       absolute intensities and desmeared errors from the
%                       specified data file
% 
% Syntax:  [scatterVector,absIntensity, desmearedError] = ...
%                                   loadStandardFile(fileName, pathName)
%
% Inputs:
%    fileName - name of the scatter data file
%    pathName - path of the scatter data file
%    Note:  The data file is expected to have the ending '.phg',
%           the header of the file are comment lines marked by '#' with the
%           first line being '# Desmeared data file from Kratky-Camera:';       
%           after the comments a table of five columns follows, the second
%           column contains the scatter vectors, the fourth column contains
%           the corresponding absolute intensities, and the fifth column 
%			contains the corresponding desmeared errors.
%
% Outputs:
%    scatterVector - scatter vectors in 1/nm from the data file
%    absIntensity - absolute intensities in e.u./nm^3 corresponding to the
%                   scatter vectors
%    desmearedError - 
%

% Other m-files required: none
% Subfunctions: none
% MAT-files required: none
%
% Variables: 	var1: output variable
% 				var2: ...
% 				var3: ...
%
% Known bugs: none
%
% Author: Susanne Pieschner
% Last revision: 13-April-2015

%%

% open data file (with read-only permission 'r')
% check whether the first line reads as expected
% and count the number of comment lines
dataFile =  fopen([pathName fileName],'r');
tline = fgets(dataFile); % extract line by line
expectedHeadline ='# Desmeared data file from Kratky-Camera:';
assert(~isempty(tline),'errMsg:FileEmpty','The selected data file is empty!')
if ~strncmp(tline, expectedHeadline, length(expectedHeadline))
    warndlg(['The data file does not begin as expected' ...
    '(with ''# Desmeared data file from Kratky-Camera:''). '...
    'Please make sure that the data file has the required format!'],...
    'errMsg:firstLine')
end

nHeaderLines = 0; % number of comment header lines in the file
while (tline(1)=='#')
    nHeaderLines = nHeaderLines + 1;
    tline = fgets(dataFile);
end

% the function importdata() loads data from data file, reading numeric 
% data separated by delimiter starting from line nHeaderLines+1;
% it returns a structure containing the cell array textdata and array data
% of numerical values
delimiter = ' '; % column separator character
impData = importdata([pathName fileName], delimiter, nHeaderLines);
if (size(impData.data,2)~=5)
    error('errMsg:wrongFormat',['The selected data file does not have'...
            'the required format!'])
else
    scatterVector = impData.data(:,2);
    absIntensity = impData.data(:,4);
    desmearedError = impData.data(:,5);
end
end

