%     This file is part of correlation_gui.
% 
%     correlation_gui is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     correlation_gui is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with correlation_gui.  If not, see <http://www.gnu.org/licenses/>.
% 
%     Diese Datei ist Teil von correlation_gui.
% 
%     correlation_gui ist Freie Software: Sie knnen es unter den Bedingungen
%     der GNU General Public License, wie von der Free Software Foundation,
%     Version 3 der Lizenz oder jeder neueren verffentlichten Version, 
%     weiter verteilen und/oder modifizieren.
% 
%     correlation_gui wird in der Hoffnung, dass es ntzlich sein wird, aber
%     OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
%     Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
%     Siehe die GNU General Public License fr weitere Details.
% 
%     Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
%     Programm erhalten haben. Wenn nicht, siehe <https://www.gnu.org/licenses/>.

function fcnValues = modelFunction(param, s1, P, sigma, minDist, resolWidth)
% modelFunction -  computes the values of the model function 
%                   (cf. Masterarbeit Anne Seidlitz p. 28  K''_sim(s) )
%                   
% 
% Syntax: fcnValues = modelFunction(param, s, P, sigma, minDist, resolWidth)
%
% Inputs:
%    param - parameters sigma_a, sigma_c, d_a, d_c of the model function
%    s1 - (equidistant) values of the scatter vectors in 1/nm 
%    P - Porod parameter of a given Porod Fit (I = P*s^(-4) + c1*s^2 + c2)
%    sigma - width of the Gaussian window function 
%               exp((2*pi*sigma.*scatterVector)^2)
%    minDist - distance between equidistant scatter vectors in
%               "low-resolution"
%   resolWidth - resolution width used for the convultion
%
% Outputs:
%    fcnValues - values of the model function at the points given by s

% Other m-files required: none
% Subfunctions: none
% MAT-files required: none

% Known bugs: none
%
% Author: Susanne Pieschner
% Last revision: 17-June-2015

%%

stepSize =  abs(min(diff(s1))); % distance between scatter vectors in "high-resolution"

% s1=0 has to be treated seperately, as the model function is not defined
% in that point

s = [s1(1:floor(end/2)); s1(ceil(end/2)+1:end)]; %excludes zero

Res = 16.*pi^3.*P...
    ./(1 - 2.*exp(-2.*(pi.*s).^2.*((param(1))^2+(param(2))^2))...
    	.*cos(2.*pi.*s.*(param(3)+param(4)))+exp(-4.*((pi.*s).^2)*((param(1))^2+(param(2))^2)))...
    .*(exp(-2.*((pi.*s.*param(1)).^2)).*cos(2.*pi.*s.*param(3))...
        + exp(-2.*((pi.*s.*param(2)).^2)).*cos(2.*pi.*s.*param(4))...
        - 2.*exp(-2.*(pi.*s).^2.*((param(1))^2+(param(2))^2)).*cos(2.*pi.*s.*(param(3)+param(4)))...
        - exp(-4.*((pi.*s.*param(1)).^2)) .*exp(-2.*(pi.*s.*param(2)).^2) .*cos(2.*pi.*s.*param(4))...
        - exp(-4.*((pi.*s.*param(2)).^2)) .*exp(-2.*(pi.*s.*param(1)).^2) .*cos(2.*pi.*s.*param(3))...
        + 2.*exp(-4.*(pi.*s).^2*(param(1)^2+param(2)^2)))...
    .*exp(-(2.*sigma.*pi.*s).^2);

% Model function is set to 16*pi^3*P at zero
Res1 = [Res(1:floor(end/2)); 16.*pi^3.*P; Res(ceil(end/2)+1:end)];

z = (1./(resolWidth.*sqrt(2.*pi))).*exp(-s1.^2./(2*resolWidth.^2));

fcnValues = conv(Res1,z,'same')*stepSize;
end