%     This file is part of correlation_gui.
% 
%     correlation_gui is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     any later version.
% 
%     correlation_gui is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with correlation_gui.  If not, see <http://www.gnu.org/licenses/>.
% 
%     Diese Datei ist Teil von correlation_gui.
% 
%     correlation_gui ist Freie Software: Sie knnen es unter den Bedingungen
%     der GNU General Public License, wie von der Free Software Foundation,
%     Version 3 der Lizenz oder jeder neueren verffentlichten Version, 
%     weiter verteilen und/oder modifizieren.
% 
%     correlation_gui wird in der Hoffnung, dass es ntzlich sein wird, aber
%     OHNE JEDE GEWHRLEISTUNG, bereitgestellt; sogar ohne die implizite
%     Gewhrleistung der MARKTFHIGKEIT oder EIGNUNG FR EINEN BESTIMMTEN ZWECK.
%     Siehe die GNU General Public License fr weitere Details.
% 
%     Sie sollten eine Kopie der GNU General Public License zusammen mit diesem
%     Programm erhalten haben. Wenn nicht, siehe <https://www.gnu.org/licenses/>.

function [P, C1, C2] = porodFit(scattVect, absInt, minIndex, maxIndex,...
                          sigma, startValuesP,isFixedP, fixedValueP,...
                          isFixedC1, fixedValueC1, isFixedC2, fixedValueC2)
% porodFit -    Calculates the parameters P, c1, c2 from fitting the data
%               absInt and scattVect to the function 
%                   absInt(s) = P*scattVect^(-4) + c1*scattVect^2 + c2;
%               Each of the parameters can be chosen to be fixed at a
%               specified  value;
%               If the Porod parameter is not fixed, 4 starting values for 
%               P are required.
%               Fitting the function to the data is a linear least square 
%               problem which is solved using the backslash opeartor \.
% 
% Syntax:  [P, C1, C2] = porodFit_v4(scattVect, absInt, minIndex,...
%                          maxIndex, sigma, startValuesP,isFixedP, fixedP,...
%                          isFixedC1, fixedC1, isFixedC2, fixedC2)
%
% Inputs:
%       scattVect - scatter vectors in 1/nm from the data file
%       absInt - absolute intensities in e.u./nm^3 corresponding to the
%                   scatter vectors
%       minIndex, maxIndex - index range for the Porod Fit
%       sigma - width of the gaussain window function
%       startValuesP
%       isFixedP
%       fixedValueP
%       isFixedC1
%       fixedValueC1
%       isFixedC2
%       fixedValueC2
%
% Outputs:
%    P - 
%    C1 - 
%    C2 - 
%
%

% Other m-files required: calculateK2AtZero.m, 
% Subfunctions: none
% MAT-files required: none
%
% Known bugs: none
%
% Author: Susanne Pieschner
% Last revision: 12-August-2015

%%

% set parameters to given values, subject to change if ifFixed* variables
% are equal to 1
P = fixedValueP;
C1 = fixedValueC1;
C2 = fixedValueC2;

% Perform the least square fit according to the specified case.
% For the switch expression, convert the binary string composed of the 2
% isFixedC* binary variables into a decimal number (there are 2^2=4 cases).
% The fitting is performed by varParam = coeffMatrix \ RHS, where
% varParam are the variable parameters that need to be determined,
% coeffMatrix is the coefficient matrix with dimensions= number of data
% points x dim(varParam),
% and RHS is the right hand side vector with length = number of data points
switchExpression = bin2dec(num2str([isFixedC1 isFixedC2]));

 
% For P not fixed, the function is fitted with the additional condition 
% K''(0)=0. To this end, the function is first fitted for each of the 
% specified starting values of P and the corresponding values of K''(0) are
% calculated. Then, a linear fit is performed for the data points 
% (P_i, K''(0)_i) (i=1,...4) in order to determine the zero P0 of this 
% fitted function. Finally, the function  is fitted to the data with P
% fixed at P0.
% For the fitting only the data points in the range minIndex to maxIndex
% are used; while for the calculation of K''(0) all data points are used
if (~isFixedP)
    % allocate storage for the values of K''(0) for the different starting
    % values of P
    K2ZeroVector = NaN(1,length(startValuesP));
    
    switch switchExpression
        case 0 % all 3 parameters vary
            coeffMatrix = [scattVect(minIndex:maxIndex).^2 ...
                         ones(length(scattVect(minIndex:maxIndex)),1)];
            for i = 1:length(startValuesP)
                P = startValuesP(i);
                RHS = absInt(minIndex:maxIndex) - ...
                        P * scattVect(minIndex:maxIndex).^(-4);
                varParam = coeffMatrix \ RHS;
                C1 = varParam(1);
                C2 = varParam(2);
                K2ZeroVector(i) = calculateK2AtZero(scattVect, absInt, ... 
                                    sigma, P, C1, C2);
            end
        case 1 % c2 is fixed; P and c1 vary
            coeffMatrix = scattVect(minIndex:maxIndex).^2;
            for i = 1:length(startValuesP)
                P = startValuesP(i);
                RHS = absInt(minIndex:maxIndex) - ...
                        P * scattVect(minIndex:maxIndex).^(-4) - C2;
                varParam = coeffMatrix \ RHS;
                C1 = varParam;
                K2ZeroVector(i) = calculateK2AtZero(scattVect, absInt, ... 
                                    sigma, P, C1, C2);
            end
        case 2 % c1 is fixed; P and c2 vary
            coeffMatrix = ones(length(scattVect(minIndex:maxIndex)),1);
            for i = 1:length(startValuesP)
                P = startValuesP(i);
                RHS = absInt(minIndex:maxIndex) - ...
                    P * scattVect(minIndex:maxIndex).^(-4) - ...
                    C1 * scattVect(minIndex:maxIndex).^2;
                varParam = coeffMatrix \ RHS;
                C2 = varParam;
                K2ZeroVector(i) = calculateK2AtZero(scattVect, absInt, ... 
                                    sigma, P, C1, C2);
            end
        case 3 % c2 and c1 are fixed; P varies
            for i = 1:length(startValuesP)
                P = startValuesP(i);
                K2ZeroVector(i) = calculateK2AtZero(scattVect, absInt, ... 
                                    sigma, P, C1, C2);
            end
    end
    % Determine P such that K''(0)=0 using linear regression
    coeff = polyfit(K2ZeroVector, startValuesP, 1);
    P = polyval(coeff,0);
end

    
% P either was fixed or has been determined above    
switch switchExpression
    case 0 % P is fixed; c1 and c2 vary
        RHS = absInt(minIndex:maxIndex) - ...
            P * scattVect(minIndex:maxIndex).^(-4);
        coeffMatrix = [scattVect(minIndex:maxIndex).^2 ...
                        ones(length(scattVect(minIndex:maxIndex)),1)];
        varParam = coeffMatrix \ RHS;
        C1 = varParam(1);
        C2 = varParam(2);
    case 1 % P and c2 fixed; c1 varies
        RHS = absInt(minIndex:maxIndex) - ...
            P * scattVect(minIndex:maxIndex).^(-4) - C2;
        coeffMatrix = scattVect(minIndex:maxIndex).^2;
        varParam = coeffMatrix \ RHS;
        C1 = varParam;
    case 2 % P and c1 fixed; c2 varies
        RHS = absInt(minIndex:maxIndex) - ...
            P * scattVect(minIndex:maxIndex).^(-4) - ...
            C1 * scattVect(minIndex:maxIndex).^2;
        coeffMatrix = ones(length(scattVect(minIndex:maxIndex)),1);
        varParam = coeffMatrix \ RHS;
        C2 = varParam;
    case 3 % all 3 parameters are fixed
        % nothing to do
end

